﻿using System;
using System.Collections.Generic;
namespace ListWithChangedEvent
{
    using System.Collections;

    // A delegate type for hooking up change notifications.
    public delegate void ChangedEventHandler(object sender, EventArgs e);

    // A class that works just like ArrayList, but sends event
    // notifications whenever the list changes.
    public class ListWithChangedEvent<T> : List<T>
    {

        public ListWithChangedEvent() { }

        public ListWithChangedEvent(ListWithChangedEvent<T> toAdd){
            this.AddRange(toAdd);
        }
        // An event that clients can use to be notified whenever the
        // elements of the list change.
        public event ChangedEventHandler Changed;
        

        // Invoke the Changed event; called whenever list changes
        protected virtual void OnChanged(EventArgs e)
        {
            if (Changed != null)
                Changed(this, e);
        }

        // Override some of the methods that can change the list;
        // invoke event after each
        
        public new void Add(T value)
        {
            base.Add(value);
            OnChanged(EventArgs.Empty);
            //return i;
        }

        public new void Clear()
        {
            base.Clear();
            OnChanged(EventArgs.Empty);
        }

        public new T this[int index]
        {
            set
            {
                base[index] = value;
                OnChanged(EventArgs.Empty);
            }
            get
            {
                //OnChanged(EventArgs.Empty);
                return base[index];
                
            }
        }
    }
}

namespace TestEvents
{
    using ListWithChangedEvent;

    class EventListener
    {
        private ListWithChangedEvent<object> List;

        public EventListener(ListWithChangedEvent<object> list)
        {
            List = list;
            // Add "ListChanged" to the Changed event on "List".
            List.Changed += new ChangedEventHandler(ListChanged);
        }

        // This will be called whenever the list changes.
        private void ListChanged(object sender, EventArgs e)
        {
            Console.WriteLine("This is called when the event fires.");
        }

        public void Detach()
        {
            // Detach the event and delete the list
            List.Changed -= new ChangedEventHandler(ListChanged);
            List = null;
        }
    }

}
